@@ -26,6 +26,9 @@ namespace xp
         offsetof(O, M)                                             \
         _Pragma("clang diagnostic pop")
     #else
+        #ifdef __GNUC__
+            #pragma GCC diagnostic ignored "-Winvalid-offsetof"
+        #endif
         #define xoffsetof(O, M) offsetof(O, M)
     #endif
 
@@ -44,7 +47,7 @@ namespace xp
         using derived_type = D;
 
         derived_type& derived_cast() & noexcept;
-        const derived_type& derived_cast() const& noexcept;
+        const derived_type& derived_cast() const & noexcept;
         derived_type derived_cast() && noexcept;
 
         using value_type = T;
@@ -71,9 +74,9 @@ namespace xp
         value_type m_value;
     };
 
-    /*******************
-     * XPROPERTY macro *
-     *******************/
+    /********************************************************
+     * XPROPERTY, XDEFAULT_VALUE, XDEFAULT_GENERATOR macros *
+     ********************************************************/
 
     // XPROPERTY(Type, Owner, Name)
     //
@@ -81,35 +84,50 @@ namespace xp
     //
     // The owner type must have two template methods
     //
-    //  - invoke_validators<std::size_t Offset, typename const_ref>( const_ref value);
+    //  - invoke_validators<std::size_t Offset, typename const_ref>(const_ref value);
     //  - invoke_observers<std::size_t Offset>();
     //
     // The `Offset` integral parameter is the offset of the observed member in the owner class.
     // The `const_ref` typename is a constant reference type on the proposed value.
 
-    #define XPROPERTY(T, O, D)                                                              \
-    class D##_property : public ::xp::xproperty<T, O, D##_property>                         \
-    {                                                                                       \
-    public:                                                                                 \
-                                                                                            \
-        template <class V>                                                                  \
-        inline typename ::xp::xproperty<T, O, D##_property>::reference operator=(V&& value) \
-        {                                                                                   \
-            return ::xp::xproperty<T, O, D##_property>::operator=(std::forward<V>(value));  \
-        }                                                                                   \
-                                                                                            \
-        static inline const std::string& name() noexcept                                    \
-        {                                                                                   \
-            static const std::string name = #D;                                             \
-            return name;                                                                    \
-        }                                                                                   \
-                                                                                            \
-        static inline constexpr std::size_t offset() noexcept                               \
-        {                                                                                   \
-            return xoffsetof(O, D);                                                         \
-        }                                                                                   \
+    #define XPROPERTY_DEFAULT(T, O, D, value)                                                    \
+    class D##_property : public ::xp::xproperty<T, O, D##_property>                              \
+    {                                                                                            \
+    public:                                                                                      \
+                                                                                                 \
+        using ::xp::xproperty<T, O, D##_property>::operator=;                                    \
+                                                                                                 \
+        static inline const std::string& name() noexcept                                         \
+        {                                                                                        \
+            static const std::string name = #D;                                                  \
+            return name;                                                                         \
+        }                                                                                        \
+                                                                                                 \
+        static inline constexpr std::size_t offset() noexcept                                    \
+        {                                                                                        \
+            return xoffsetof(O, D);                                                              \
+        }                                                                                        \
+                                                                                                 \
+        static inline T default_value()                                                          \
+        {                                                                                        \
+            return T(value);                                                                     \
+        }                                                                                        \
     } D;
 
+    #define XPROPERTY_NODEFAULT(T, O, D)                                                         \
+    XPROPERTY_DEFAULT(T, O, D, T())
+
+    #define XPROPERTY_OVERLOAD(_1, _2, _3, _4, NAME, ...) NAME
+
+    #ifdef _MSC_VER
+    // Workaround for MSVC not expanding macros
+    #define XPROPERTY_EXPAND( x ) x
+    #define XPROPERTY(...) XPROPERTY_EXPAND(XPROPERTY_OVERLOAD(__VA_ARGS__, XPROPERTY_DEFAULT, XPROPERTY_NODEFAULT)(__VA_ARGS__))
+    #else
+    #define XPROPERTY(...) XPROPERTY_OVERLOAD(__VA_ARGS__, XPROPERTY_DEFAULT, XPROPERTY_NODEFAULT)(__VA_ARGS__)
+    #endif
+
+
     /***********************
      * MAKE_OBSERVED macro *
      ***********************/
@@ -121,8 +139,10 @@ namespace xp
     #define MAKE_OBSERVED()                                                                 \
     template <class P>                                                                      \
     inline void notify(const P&) const {}                                                   \
+                                                                                            \
     template <class P>                                                                      \
     inline void invoke_observers() const {}                                                 \
+                                                                                            \
     template <class P, class V>                                                             \
     inline auto invoke_validators(V&& r) const { return r; }
 
@@ -174,7 +194,7 @@ namespace xp
 
     template <class T, class O, class D>
     inline constexpr xproperty<T, O, D>::xproperty() noexcept(noexcept(std::is_nothrow_constructible<value_type>::value))
-        : m_value()
+        : m_value(D::default_value())
     {
     }
 
